# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from knack.help_files import helps  # pylint: disable=unused-import

helps['login'] = """
type: command
short-summary: Log in to Azure.
long-summary: >-
    By default, this command logs in with a user account. CLI will try to launch a web browser to log in interactively.
    If a web browser is not available, CLI will fall back to device code login.

    To login with a service principal, specify --service-principal.
examples:
    - name: Log in interactively.
      text: az login
    - name: Log in with user name and password. This doesn't work with Microsoft accounts or accounts that have two-factor authentication enabled. Use -p=secret if the first character of the password is '-'.
      text: az login -u johndoe@contoso.com -p VerySecret
    - name: Log in with a service principal using client secret. Use -p=secret if the first character of the password is '-'.
      text: az login --service-principal -u http://azure-cli-2016-08-05-14-31-15 -p VerySecret --tenant contoso.onmicrosoft.com
    - name: Log in with a service principal using client certificate.
      text: az login --service-principal -u http://azure-cli-2016-08-05-14-31-15 -p ~/mycertfile.pem --tenant contoso.onmicrosoft.com
    - name: Log in using a VM's system-assigned managed identity.
      text: az login --identity
    - name: Log in using a VM's user-assigned managed identity. Client or object ids of the service identity also work.
      text: az login --identity -u /subscriptions/<subscriptionId>/resourcegroups/myRG/providers/Microsoft.ManagedIdentity/userAssignedIdentities/myID
"""

helps['account'] = """
type: group
short-summary: Manage Azure subscription information.
"""

helps['account clear'] = """
type: command
short-summary: Clear all subscriptions from the CLI's local cache.
long-summary: To clear the current subscription, use 'az logout'.
"""

helps['account list'] = """
type: command
short-summary: >-
    Get a list of subscriptions for the logged in account. By default, only 'Enabled' subscriptions from the current
    cloud is shown.
"""

helps['account list-locations'] = """
type: command
short-summary: List supported regions for the current subscription.
"""

helps['account show'] = """
type: command
short-summary: Get the details of a subscription.
long-summary: If no subscription is specified, shows the current subscription.
"""

helps['account set'] = """
type: command
short-summary: Set a subscription to be the current active subscription.
"""

helps['account show'] = """
type: command
short-summary: Get the details of a subscription.
long-summary: If the subscription isn't specified, shows the details of the default subscription.
"""

helps['account get-access-token'] = """
type: command
short-summary: Get a token for utilities to access Azure.
long-summary: >
    The token will be valid for at least 5 minutes with the maximum at 60 minutes.
    If the subscription argument isn't specified, the current account is used.
examples:
    - name: Get an access token for the current account
      text: >
        az account get-access-token
    - name: Get an access token for a specific subscription
      text: >
        az account get-access-token --subscription 00000000-0000-0000-0000-000000000000
    - name: Get an access token for a specific tenant
      text: >
        az account get-access-token --tenant 00000000-0000-0000-0000-000000000000
    - name: Get an access token to use with MS Graph API
      text: >
        az account get-access-token --resource-type ms-graph
"""

helps['self-test'] = """
type: command
short-summary: Runs a self-test of the CLI.
"""



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines
# pylint: disable=unused-argument

from knack.util import CLIError
from azure.cli.core.util import sdk_no_wait


def synapse_kusto_operation_list(client):
    return client.list()


def synapse_kusto_pool_list_sku(client):
    return client.list_skus()


def synapse_kusto_pool_list(client,
                            resource_group_name,
                            workspace_name):
    return client.list_by_workspace(resource_group_name=resource_group_name,
                                    workspace_name=workspace_name)


def synapse_kusto_pool_show(client,
                            workspace_name,
                            kusto_pool_name,
                            resource_group_name):
    return client.get(workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      resource_group_name=resource_group_name)


def synapse_kusto_pool_delete(client,
                              workspace_name,
                              resource_group_name,
                              kusto_pool_name,
                              no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       workspace_name=workspace_name,
                       resource_group_name=resource_group_name,
                       kusto_pool_name=kusto_pool_name)


def synapse_kusto_pool_list_follower_database(client,
                                              workspace_name,
                                              kusto_pool_name,
                                              resource_group_name):
    return client.list_follower_databases(workspace_name=workspace_name,
                                          kusto_pool_name=kusto_pool_name,
                                          resource_group_name=resource_group_name)


def synapse_kusto_pool_list_language_extension(client,
                                               workspace_name,
                                               kusto_pool_name,
                                               resource_group_name):
    return client.list_language_extensions(workspace_name=workspace_name,
                                           kusto_pool_name=kusto_pool_name,
                                           resource_group_name=resource_group_name)


def synapse_kusto_pool_list_sku(client,
                                workspace_name,
                                kusto_pool_name,
                                resource_group_name):
    return client.list_skus_by_resource(workspace_name=workspace_name,
                                        kusto_pool_name=kusto_pool_name,
                                        resource_group_name=resource_group_name)


def synapse_kusto_pool_start(client,
                             workspace_name,
                             kusto_pool_name,
                             resource_group_name,
                             no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_start,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_pool_stop(client,
                            workspace_name,
                            kusto_pool_name,
                            resource_group_name,
                            no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_stop,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_attached_database_configuration_list(client,
                                                       workspace_name,
                                                       kusto_pool_name,
                                                       resource_group_name):
    return client.list_by_kusto_pool(workspace_name=workspace_name,
                                     kusto_pool_name=kusto_pool_name,
                                     resource_group_name=resource_group_name)


def synapse_kusto_attached_database_configuration_show(client,
                                                       workspace_name,
                                                       kusto_pool_name,
                                                       attached_database_configuration_name,
                                                       resource_group_name):
    return client.get(workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      attached_database_configuration_name=attached_database_configuration_name,
                      resource_group_name=resource_group_name)


def synapse_kusto_attached_database_configuration_create(client,
                                                         workspace_name,
                                                         kusto_pool_name,
                                                         attached_database_configuration_name,
                                                         resource_group_name,
                                                         location=None,
                                                         database_name=None,
                                                         kusto_pool_resource_id=None,
                                                         default_principals_modification_kind=None,
                                                         table_level_sharing_properties=None,
                                                         no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    if database_name is not None:
        parameters['database_name'] = database_name
    if kusto_pool_resource_id is not None:
        parameters['kusto_pool_resource_id'] = kusto_pool_resource_id
    if default_principals_modification_kind is not None:
        parameters['default_principals_modification_kind'] = default_principals_modification_kind
    if table_level_sharing_properties is not None:
        parameters['table_level_sharing_properties'] = table_level_sharing_properties
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       attached_database_configuration_name=attached_database_configuration_name,
                       resource_group_name=resource_group_name,
                       parameters=parameters)


def synapse_kusto_attached_database_configuration_update(instance,
                                                         workspace_name,
                                                         kusto_pool_name,
                                                         attached_database_configuration_name,
                                                         resource_group_name,
                                                         location=None,
                                                         database_name=None,
                                                         kusto_pool_resource_id=None,
                                                         default_principals_modification_kind=None,
                                                         table_level_sharing_properties=None,
                                                         no_wait=False):
    if location is not None:
        instance.location = location
    if database_name is not None:
        instance.database_name = database_name
    if kusto_pool_resource_id is not None:
        instance.kusto_pool_resource_id = kusto_pool_resource_id
    if default_principals_modification_kind is not None:
        instance.default_principals_modification_kind = default_principals_modification_kind
    if table_level_sharing_properties is not None:
        instance.table_level_sharing_properties = table_level_sharing_properties
    return instance


def synapse_kusto_attached_database_configuration_delete(client,
                                                         workspace_name,
                                                         kusto_pool_name,
                                                         attached_database_configuration_name,
                                                         resource_group_name,
                                                         no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       attached_database_configuration_name=attached_database_configuration_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_database_list(client,
                                resource_group_name,
                                workspace_name,
                                kusto_pool_name):
    return client.list_by_kusto_pool(resource_group_name=resource_group_name,
                                     workspace_name=workspace_name,
                                     kusto_pool_name=kusto_pool_name)


def synapse_kusto_database_show(client,
                                resource_group_name,
                                workspace_name,
                                kusto_pool_name,
                                database_name):
    return client.get(resource_group_name=resource_group_name,
                      workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      database_name=database_name)


def synapse_kusto_database_create(client,
                                  resource_group_name,
                                  workspace_name,
                                  kusto_pool_name,
                                  database_name,
                                  read_write_database=None,
                                  no_wait=False):
    all_parameters = []
    if read_write_database is not None:
        all_parameters.append(read_write_database)
    if len(all_parameters) > 1:
        raise CLIError('at most one of  read_write_database is needed for parameters!')
    if len(all_parameters) != 1:
        raise CLIError('parameters is required. but none of read_write_database is provided!')
    parameters = all_parameters[0] if len(all_parameters) == 1 else None
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       parameters=parameters)


def synapse_kusto_database_update(client,
                                  resource_group_name,
                                  workspace_name,
                                  kusto_pool_name,
                                  database_name,
                                  read_write_database=None,
                                  no_wait=False):
    all_parameters = []
    if read_write_database is not None:
        all_parameters.append(read_write_database)
    if len(all_parameters) > 1:
        raise CLIError('at most one of  read_write_database is needed for parameters!')
    if len(all_parameters) != 1:
        raise CLIError('parameters is required. but none of read_write_database is provided!')
    parameters = all_parameters[0] if len(all_parameters) == 1 else None
    return sdk_no_wait(no_wait,
                       client.begin_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       parameters=parameters)


def synapse_kusto_database_delete(client,
                                  resource_group_name,
                                  workspace_name,
                                  kusto_pool_name,
                                  database_name,
                                  no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name)


def synapse_kusto_data_connection_list(client,
                                       resource_group_name,
                                       workspace_name,
                                       kusto_pool_name,
                                       database_name):
    return client.list_by_database(resource_group_name=resource_group_name,
                                   workspace_name=workspace_name,
                                   kusto_pool_name=kusto_pool_name,
                                   database_name=database_name)


def synapse_kusto_data_connection_show(client,
                                       resource_group_name,
                                       workspace_name,
                                       kusto_pool_name,
                                       database_name,
                                       data_connection_name):
    return client.get(resource_group_name=resource_group_name,
                      workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      database_name=database_name,
                      data_connection_name=data_connection_name)


def synapse_kusto_data_connection_event_grid_create(client,
                                                    resource_group_name,
                                                    workspace_name,
                                                    kusto_pool_name,
                                                    database_name,
                                                    data_connection_name,
                                                    location=None,
                                                    storage_account_resource_id=None,
                                                    event_hub_resource_id=None,
                                                    consumer_group=None,
                                                    table_name=None,
                                                    mapping_rule_name=None,
                                                    data_format=None,
                                                    ignore_first_record=None,
                                                    blob_storage_event_type=None,
                                                    no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'EventGrid'
    if storage_account_resource_id is not None:
        parameters['storage_account_resource_id'] = storage_account_resource_id
    if event_hub_resource_id is not None:
        parameters['event_hub_resource_id'] = event_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if ignore_first_record is not None:
        parameters['ignore_first_record'] = ignore_first_record
    if blob_storage_event_type is not None:
        parameters['blob_storage_event_type'] = blob_storage_event_type
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_event_hub_create(client,
                                                   resource_group_name,
                                                   workspace_name,
                                                   kusto_pool_name,
                                                   database_name,
                                                   data_connection_name,
                                                   location=None,
                                                   event_hub_resource_id=None,
                                                   consumer_group=None,
                                                   table_name=None,
                                                   mapping_rule_name=None,
                                                   data_format=None,
                                                   event_system_properties=None,
                                                   compression=None,
                                                   managed_identity_resource_id=None,
                                                   no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'EventHub'
    if event_hub_resource_id is not None:
        parameters['event_hub_resource_id'] = event_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if event_system_properties is not None:
        parameters['event_system_properties'] = event_system_properties
    if compression is not None:
        parameters['compression'] = compression
    if managed_identity_resource_id is not None:
        parameters['managed_identity_resource_id'] = managed_identity_resource_id
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_iot_hub_create(client,
                                                 resource_group_name,
                                                 workspace_name,
                                                 kusto_pool_name,
                                                 database_name,
                                                 data_connection_name,
                                                 location=None,
                                                 iot_hub_resource_id=None,
                                                 consumer_group=None,
                                                 table_name=None,
                                                 mapping_rule_name=None,
                                                 data_format=None,
                                                 event_system_properties=None,
                                                 shared_access_policy_name=None,
                                                 no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'IotHub'
    if iot_hub_resource_id is not None:
        parameters['iot_hub_resource_id'] = iot_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if event_system_properties is not None:
        parameters['event_system_properties'] = event_system_properties
    if shared_access_policy_name is not None:
        parameters['shared_access_policy_name'] = shared_access_policy_name
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_event_grid_update(client,
                                                    resource_group_name,
                                                    workspace_name,
                                                    kusto_pool_name,
                                                    database_name,
                                                    data_connection_name,
                                                    location=None,
                                                    storage_account_resource_id=None,
                                                    event_hub_resource_id=None,
                                                    consumer_group=None,
                                                    table_name=None,
                                                    mapping_rule_name=None,
                                                    data_format=None,
                                                    ignore_first_record=None,
                                                    blob_storage_event_type=None,
                                                    no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'EventGrid'
    if storage_account_resource_id is not None:
        parameters['storage_account_resource_id'] = storage_account_resource_id
    if event_hub_resource_id is not None:
        parameters['event_hub_resource_id'] = event_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if ignore_first_record is not None:
        parameters['ignore_first_record'] = ignore_first_record
    if blob_storage_event_type is not None:
        parameters['blob_storage_event_type'] = blob_storage_event_type
    return sdk_no_wait(no_wait,
                       client.begin_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_event_hub_update(client,
                                                   resource_group_name,
                                                   workspace_name,
                                                   kusto_pool_name,
                                                   database_name,
                                                   data_connection_name,
                                                   location=None,
                                                   event_hub_resource_id=None,
                                                   consumer_group=None,
                                                   table_name=None,
                                                   mapping_rule_name=None,
                                                   data_format=None,
                                                   event_system_properties=None,
                                                   compression=None,
                                                   managed_identity_resource_id=None,
                                                   no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'EventHub'
    if event_hub_resource_id is not None:
        parameters['event_hub_resource_id'] = event_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if event_system_properties is not None:
        parameters['event_system_properties'] = event_system_properties
    if compression is not None:
        parameters['compression'] = compression
    if managed_identity_resource_id is not None:
        parameters['managed_identity_resource_id'] = managed_identity_resource_id
    return sdk_no_wait(no_wait,
                       client.begin_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_iot_hub_update(client,
                                                 resource_group_name,
                                                 workspace_name,
                                                 kusto_pool_name,
                                                 database_name,
                                                 data_connection_name,
                                                 location=None,
                                                 iot_hub_resource_id=None,
                                                 consumer_group=None,
                                                 table_name=None,
                                                 mapping_rule_name=None,
                                                 data_format=None,
                                                 event_system_properties=None,
                                                 shared_access_policy_name=None,
                                                 no_wait=False):
    parameters = {}
    if location is not None:
        parameters['location'] = location
    parameters['kind'] = 'IotHub'
    if iot_hub_resource_id is not None:
        parameters['iot_hub_resource_id'] = iot_hub_resource_id
    if consumer_group is not None:
        parameters['consumer_group'] = consumer_group
    if table_name is not None:
        parameters['table_name'] = table_name
    if mapping_rule_name is not None:
        parameters['mapping_rule_name'] = mapping_rule_name
    if data_format is not None:
        parameters['data_format'] = data_format
    if event_system_properties is not None:
        parameters['event_system_properties'] = event_system_properties
    if shared_access_policy_name is not None:
        parameters['shared_access_policy_name'] = shared_access_policy_name
    return sdk_no_wait(no_wait,
                       client.begin_update,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name,
                       parameters=parameters)


def synapse_kusto_data_connection_delete(client,
                                         resource_group_name,
                                         workspace_name,
                                         kusto_pool_name,
                                         database_name,
                                         data_connection_name,
                                         no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       resource_group_name=resource_group_name,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       data_connection_name=data_connection_name)


def synapse_kusto_pool_principal_assignment_list(client,
                                                 workspace_name,
                                                 kusto_pool_name,
                                                 resource_group_name):
    return client.list(workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_pool_principal_assignment_show(client,
                                                 workspace_name,
                                                 kusto_pool_name,
                                                 principal_assignment_name,
                                                 resource_group_name):
    return client.get(workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      principal_assignment_name=principal_assignment_name,
                      resource_group_name=resource_group_name)


def synapse_kusto_pool_principal_assignment_create(client,
                                                   workspace_name,
                                                   kusto_pool_name,
                                                   principal_assignment_name,
                                                   resource_group_name,
                                                   principal_id=None,
                                                   role=None,
                                                   tenant_id=None,
                                                   principal_type=None,
                                                   no_wait=False):
    parameters = {}
    if principal_id is not None:
        parameters['principal_id'] = principal_id
    if role is not None:
        parameters['role'] = role
    if tenant_id is not None:
        parameters['tenant_id'] = tenant_id
    if principal_type is not None:
        parameters['principal_type'] = principal_type
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       principal_assignment_name=principal_assignment_name,
                       resource_group_name=resource_group_name,
                       parameters=parameters)


def synapse_kusto_pool_principal_assignment_update(instance,
                                                   workspace_name,
                                                   kusto_pool_name,
                                                   principal_assignment_name,
                                                   resource_group_name,
                                                   principal_id=None,
                                                   role=None,
                                                   tenant_id=None,
                                                   principal_type=None,
                                                   no_wait=False):
    if principal_id is not None:
        instance.principal_id = principal_id
    if role is not None:
        instance.role = role
    if tenant_id is not None:
        instance.tenant_id = tenant_id
    if principal_type is not None:
        instance.principal_type = principal_type
    return instance


def synapse_kusto_pool_principal_assignment_delete(client,
                                                   workspace_name,
                                                   kusto_pool_name,
                                                   principal_assignment_name,
                                                   resource_group_name,
                                                   no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       principal_assignment_name=principal_assignment_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_database_principal_assignment_list(client,
                                                     workspace_name,
                                                     kusto_pool_name,
                                                     database_name,
                                                     resource_group_name):
    return client.list(workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       resource_group_name=resource_group_name)


def synapse_kusto_database_principal_assignment_show(client,
                                                     workspace_name,
                                                     kusto_pool_name,
                                                     database_name,
                                                     principal_assignment_name,
                                                     resource_group_name):
    return client.get(workspace_name=workspace_name,
                      kusto_pool_name=kusto_pool_name,
                      database_name=database_name,
                      principal_assignment_name=principal_assignment_name,
                      resource_group_name=resource_group_name)


def synapse_kusto_database_principal_assignment_create(client,
                                                       workspace_name,
                                                       kusto_pool_name,
                                                       database_name,
                                                       principal_assignment_name,
                                                       resource_group_name,
                                                       principal_id=None,
                                                       role=None,
                                                       tenant_id=None,
                                                       principal_type=None,
                                                       no_wait=False):
    parameters = {}
    if principal_id is not None:
        parameters['principal_id'] = principal_id
    if role is not None:
        parameters['role'] = role
    if tenant_id is not None:
        parameters['tenant_id'] = tenant_id
    if principal_type is not None:
        parameters['principal_type'] = principal_type
    return sdk_no_wait(no_wait,
                       client.begin_create_or_update,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       principal_assignment_name=principal_assignment_name,
                       resource_group_name=resource_group_name,
                       parameters=parameters)


def synapse_kusto_database_principal_assignment_update(instance,
                                                       workspace_name,
                                                       kusto_pool_name,
                                                       database_name,
                                                       principal_assignment_name,
                                                       resource_group_name,
                                                       principal_id=None,
                                                       role=None,
                                                       tenant_id=None,
                                                       principal_type=None,
                                                       no_wait=False):
    if principal_id is not None:
        instance.principal_id = principal_id
    if role is not None:
        instance.role = role
    if tenant_id is not None:
        instance.tenant_id = tenant_id
    if principal_type is not None:
        instance.principal_type = principal_type
    return instance


def synapse_kusto_database_principal_assignment_delete(client,
                                                       workspace_name,
                                                       kusto_pool_name,
                                                       database_name,
                                                       principal_assignment_name,
                                                       resource_group_name,
                                                       no_wait=False):
    return sdk_no_wait(no_wait,
                       client.begin_delete,
                       workspace_name=workspace_name,
                       kusto_pool_name=kusto_pool_name,
                       database_name=database_name,
                       principal_assignment_name=principal_assignment_name,
                       resource_group_name=resource_group_name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.mgmt.redhatopenshift import AzureRedHatOpenShiftClient


def cf_aro(cli_ctx, *_):
    client = get_mgmt_service_client(
        cli_ctx, AzureRedHatOpenShiftClient).open_shift_clusters

    return client



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from knack.arguments import CLIArgumentType

from azure.cli.core.commands.parameters import resource_group_name_type, tags_type


def load_arguments(self, _):

    name_arg_type = CLIArgumentType(options_list=['--name', '-n'], metavar='NAME')

    with self.argument_context('dms') as c:
        c.argument('service_name', name_arg_type, help='The name of the Service.')
        c.argument('group_name', resource_group_name_type)
        c.argument('tags', tags_type, help='A space-delimited list of tags in "tag1[=value1]" format.')

    with self.argument_context('dms project') as c:
        c.argument('service_name', options_list=['--service-name'])
        c.argument('project_name', name_arg_type, help='The name of the Project.')
        c.argument('tags', tags_type, help='A space-delimited list of tags in "tag1[=value1]" format.')

    with self.argument_context('dms project task') as c:
        c.argument('service_name', options_list=['--service-name'])
        c.argument('project_name', options_list=['--project-name'])
        c.argument('task_name', name_arg_type, help='The name of the Task.')
        c.argument('database_options',
                   help='A JSON-formatted array of objects representing database options and table mappings.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def cf_advisor(cli_ctx, **_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.advisor import AdvisorManagementClient
    return get_mgmt_service_client(cli_ctx, AdvisorManagementClient)


def advisor_mgmt_client_factory(cli_ctx, _):
    return cf_advisor(cli_ctx)


def recommendations_mgmt_client_factory(cli_ctx, _):
    return cf_advisor(cli_ctx).recommendations


def suppressions_mgmt_client_factory(cli_ctx, _):
    return cf_advisor(cli_ctx).suppressions


def configurations_mgmt_client_factory(cli_ctx, _):
    return cf_advisor(cli_ctx).configurations



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from knack.arguments import CLIArgumentType

from azure.cli.core.commands.parameters import (name_type, get_enum_type)

from .custom import (
    AzureSkuName
)


def load_arguments(self, _):

    # Kusto clusters
    sku_arg_type = CLIArgumentType(help='The name of the sku.',
                                   arg_type=get_enum_type(AzureSkuName))
    time_format_explenation = 'Duration in ISO8601 format (for example, 100 days would be P100D).'

    with self.argument_context('kusto cluster') as c:
        c.ignore('kusto_management_request_options')
        c.argument('cluster_name', arg_type=name_type, help='The name of the cluster.', id_part='name')
        c.argument('sku', arg_type=sku_arg_type)
        c.argument('capacity', type=int, help='The instance number of the VM.')

    # Kusto databases
    with self.argument_context('kusto database') as c:
        c.ignore('kusto_management_request_options')
        c.argument('cluster_name', help='The name of the cluster.', id_part='name')
        c.argument('database_name', arg_type=name_type, help='The name of the database.', id_part='child_name_1')
        c.argument('soft_delete_period', help='Amount of time that data should be kept so it is available to query. ' + time_format_explenation)
        c.argument('hot_cache_period', help='Amount of time that data should be kept in cache.' + time_format_explenation)

    # Kusto database list
    with self.argument_context('kusto database list') as c:
        c.argument('cluster_name', id_part=None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def export_artifacts(formula):
    """ Exports artifacts from the given formula. This method removes some of the properties of the
        artifact model as they do not play important part for users in create or read context.
    """
    artifacts = []
    if formula and formula.formula_content and formula.formula_content.artifacts:
        artifacts = formula.formula_content.artifacts
        for artifact in formula.formula_content.artifacts:
            del artifact.status
            del artifact.deployment_status_message
            del artifact.vm_extension_status_message
            del artifact.install_time
    return artifacts


def transform_artifact_source_list(artifact_source_list):
    return [transform_artifact_source(v) for v in artifact_source_list]


def transform_artifact_source(result):
    from collections import OrderedDict
    return OrderedDict([('name', result['name']),
                        ('sourceType', result['sourceType']),
                        ('status', result.get('status')),
                        ('uri', result.get('uri'))])


def transform_arm_template_list(arm_template_list):
    return [transform_arm_template(v) for v in arm_template_list]


def transform_arm_template(result):
    from collections import OrderedDict
    return OrderedDict([('name', result['name']),
                        ('resourceGroup', result['resourceGroup']),
                        ('publisher', result.get('publisher'))])


def transform_vm_list(vm_list):
    return [_transform_vm_dict(v) for v in vm_list]


def _transform_vm_dict(result):
    from collections import OrderedDict
    return OrderedDict([('name', result['name']),
                        ('location', result['location']),
                        ('osType', result['osType'])])


def transform_vm(result):
    from collections import OrderedDict
    return OrderedDict([('name', result.name),
                        ('location', result.location),
                        ('osType', result.os_type)])



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines

from knack.help_files import helps


helps['billing account'] = """
    type: group
    short-summary: billing account
"""

helps['billing account list'] = """
    type: command
    short-summary: "Lists the billing accounts that a user has access to."
    examples:
      - name: BillingAccountsList
        text: |-
               az billing account list
      - name: BillingAccountsListWithExpand
        text: |-
               az billing account list --expand "soldTo,billingProfiles,billingProfiles/invoiceSections"
      - name: BillingAccountsListWithExpandForEnrollmentDetails
        text: |-
               az billing account list --expand "enrollmentDetails,departments,enrollmentAccounts"
"""

helps['billing account show'] = """
    type: command
    short-summary: "Gets a billing account by its ID."
    examples:
      - name: BillingAccountWithExpand
        text: |-
               az billing account show --expand "soldTo,billingProfiles,billingProfiles/invoiceSections" --name \
"{billingAccountName}"
      - name: BillingAccounts
        text: |-
               az billing account show --name "{billingAccountName}"
"""

helps['billing account update'] = """
    type: command
    short-summary: "Updates the properties of a billing account. Currently, displayName and address can be updated. \
The operation is supported only for billing accounts with agreement type Microsoft Customer Agreement."
    parameters:
      - name: --sold-to
        short-summary: "The address of the individual or organization that is responsible for the billing account."
        long-summary: |
            Usage: --sold-to first-name=XX last-name=XX company-name=XX address-line1=XX address-line2=XX \
address-line3=XX city=XX district=XX region=XX country=XX postal-code=XX email=XX phone-number=XX

            first-name: First name.
            last-name: Last name.
            company-name: Company name.
            address-line1: Required. Address line 1.
            address-line2: Address line 2.
            address-line3: Address line 3.
            city: Address city.
            district: Address district.
            region: Address region.
            country: Required. Country code uses ISO2, 2-digit format.
            postal-code: Postal code.
            email: Email address.
            phone-number: Phone number.
    examples:
      - name: UpdateBillingAccount
        text: |-
               az billing account update --name "{billingAccountName}" --display-name "Test Account" --sold-to \
address-line1="Test Address 1" city="Redmond" company-name="Contoso" country="US" first-name="Test" last-name="User" \
postal-code="12345" region="WA"
"""

helps['billing account wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the billing account is met.
    examples:
      - name: Pause executing next line of CLI script until the billing account is successfully updated.
        text: |-
               az billing account wait --name "{billingAccountName}" --updated
"""

helps['billing balance'] = """
    type: group
    short-summary: billing balance
"""

helps['billing balance show'] = """
    type: command
    short-summary: "The available credit balance for a billing profile. This is the balance that can be used for pay \
now to settle due or past due invoices. The operation is supported only for billing accounts with agreement type \
Microsoft Customer Agreement."
    examples:
      - name: AvailableBalanceByBillingProfile
        text: |-
               az billing balance show --account-name "{billingAccountName}" --profile-name "{billingProfileName}"
"""

helps['billing instruction'] = """
    type: group
    short-summary: billing instruction
"""

helps['billing instruction list'] = """
    type: command
    short-summary: "Lists the instructions by billing profile id."
    examples:
      - name: InstructionsListByBillingProfile
        text: |-
               az billing instruction list --account-name "{billingAccountName}" --profile-name "{billingProfileName}"
"""

helps['billing instruction show'] = """
    type: command
    short-summary: "Get the instruction by name. These are custom billing instructions and are only applicable for \
certain customers."
    examples:
      - name: Instruction
        text: |-
               az billing instruction show --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--name "{instructionName}"
"""

helps['billing instruction create'] = """
    type: command
    short-summary: "Creates or updates an instruction. These are custom billing instructions and are only applicable \
for certain customers."
    examples:
      - name: PutInstruction
        text: |-
               az billing instruction create --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{instructionName}" --amount 5000 --end-date "2020-12-30T21:26:47.997Z" --start-date \
"2019-12-30T21:26:47.997Z"
"""

helps['billing profile'] = """
    type: group
    short-summary: billing profile
"""

helps['billing profile list'] = """
    type: command
    short-summary: "Lists the billing profiles that a user has access to. The operation is supported for billing \
accounts with agreement type Microsoft Customer Agreement or Microsoft Partner Agreement."
    examples:
      - name: BillingProfilesListByBillingAccount
        text: |-
               az billing profile list --account-name "{billingAccountName}"
      - name: BillingProfilesListWithExpand
        text: |-
               az billing profile list --expand "invoiceSections" --account-name "{billingAccountName}"
"""

helps['billing profile show'] = """
    type: command
    short-summary: "Gets a billing profile by its ID. The operation is supported for billing accounts with agreement \
type Microsoft Customer Agreement or Microsoft Partner Agreement."
    examples:
      - name: BillingProfile
        text: |-
               az billing profile show --account-name "{billingAccountName}" --name "{billingProfileName}"
      - name: BillingProfileWithExpand
        text: |-
               az billing profile show --expand "invoiceSections" --account-name "{billingAccountName}" --name \
"{billingProfileName}"
"""

helps['billing profile create'] = """
    type: command
    short-summary: "Creates or updates a billing profile. The operation is supported for billing accounts with \
agreement type Microsoft Customer Agreement or Microsoft Partner Agreement."
    parameters:
      - name: --bill-to
        short-summary: "Billing address."
        long-summary: |
            Usage: --bill-to first-name=XX last-name=XX company-name=XX address-line1=XX address-line2=XX \
address-line3=XX city=XX district=XX region=XX country=XX postal-code=XX email=XX phone-number=XX

            first-name: First name.
            last-name: Last name.
            company-name: Company name.
            address-line1: Required. Address line 1.
            address-line2: Address line 2.
            address-line3: Address line 3.
            city: Address city.
            district: Address district.
            region: Address region.
            country: Required. Country code uses ISO2, 2-digit format.
            postal-code: Postal code.
            email: Email address.
            phone-number: Phone number.
      - name: --enabled-azure-plans
        short-summary: "Information about the enabled azure plans."
        long-summary: |
            Usage: --enabled-azure-plans sku-id=XX

            sku-id: The sku id.

            Multiple actions can be specified by using more than one --enabled-azure-plans argument.
    examples:
      - name: CreateBillingProfile
        text: |-
               az billing profile create --account-name "{billingAccountName}" --name "{billingProfileName}" --bill-to \
address-line1="Test Address 1" city="Redmond" country="US" first-name="Test" last-name="User" postal-code="12345" \
region="WA" --display-name "Finance" --enabled-azure-plans sku-id="0001" --enabled-azure-plans sku-id="0002" \
--invoice-email-opt-in true --po-number "ABC12345"
"""

helps['billing profile update'] = """
    type: command
    short-summary: "Creates or updates a billing profile. The operation is supported for billing accounts with \
agreement type Microsoft Customer Agreement or Microsoft Partner Agreement."
    parameters:
      - name: --bill-to
        short-summary: "Billing address."
        long-summary: |
            Usage: --bill-to first-name=XX last-name=XX company-name=XX address-line1=XX address-line2=XX \
address-line3=XX city=XX district=XX region=XX country=XX postal-code=XX email=XX phone-number=XX

            first-name: First name.
            last-name: Last name.
            company-name: Company name.
            address-line1: Required. Address line 1.
            address-line2: Address line 2.
            address-line3: Address line 3.
            city: Address city.
            district: Address district.
            region: Address region.
            country: Required. Country code uses ISO2, 2-digit format.
            postal-code: Postal code.
            email: Email address.
            phone-number: Phone number.
      - name: --enabled-azure-plans
        short-summary: "Information about the enabled azure plans."
        long-summary: |
            Usage: --enabled-azure-plans sku-id=XX

            sku-id: The sku id.

            Multiple actions can be specified by using more than one --enabled-azure-plans argument.
"""

helps['billing profile wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the billing profile is met.
    examples:
      - name: Pause executing next line of CLI script until the billing profile is successfully created.
        text: |-
               az billing profile wait --expand "invoiceSections" --account-name "{billingAccountName}" --name \
"{billingProfileName}" --created
      - name: Pause executing next line of CLI script until the billing profile is successfully updated.
        text: |-
               az billing profile wait --expand "invoiceSections" --account-name "{billingAccountName}" --name \
"{billingProfileName}" --updated
"""

helps['billing customer'] = """
    type: group
    short-summary: billing customer
"""

helps['billing customer list'] = """
    type: command
    short-summary: "Lists the customers that are billed to a billing account. The operation is supported only for \
billing accounts with agreement type Microsoft Partner Agreement."
    examples:
      - name: CustomersListByBillingProfile
        text: |-
               az billing customer list --account-name "{billingAccountName}" --profile-name "{billingProfileName}"
"""

helps['billing customer show'] = """
    type: command
    short-summary: "Gets a customer by its ID. The operation is supported only for billing accounts with agreement \
type Microsoft Partner Agreement."
    examples:
      - name: Customer
        text: |-
               az billing customer show --account-name "{billingAccountName}" --name "{customerName}"
      - name: CustomerWithExpand
        text: |-
               az billing customer show --expand "enabledAzurePlans,resellers" --account-name "{billingAccountName}" \
--name "{customerName}"
"""

helps['billing invoice section'] = """
    type: group
    short-summary: billing invoice section
"""

helps['billing invoice section list'] = """
    type: command
    short-summary: "Lists the invoice sections that a user has access to. The operation is supported only for billing \
accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: InvoiceSectionsListByBillingProfile
        text: |-
               az billing invoice section list --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}"
"""

helps['billing invoice section show'] = """
    type: command
    short-summary: "Gets an invoice section by its ID. The operation is supported only for billing accounts with \
agreement type Microsoft Customer Agreement."
    examples:
      - name: InvoiceSection
        text: |-
               az billing invoice section show --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{invoiceSectionName}"
"""

helps['billing invoice section create'] = """
    type: command
    short-summary: "Creates or updates an invoice section. The operation is supported only for billing accounts with \
agreement type Microsoft Customer Agreement."
    examples:
      - name: PutInvoiceSection
        text: |-
               az billing invoice section create --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{invoiceSectionName}" --display-name "invoiceSection1" --labels costCategory="Support" \
pcCode="A123456"
"""

helps['billing invoice section update'] = """
    type: command
    short-summary: "Creates or updates an invoice section. The operation is supported only for billing accounts with \
agreement type Microsoft Customer Agreement."
"""

helps['billing invoice section wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the billing invoice section is met.
    examples:
      - name: Pause executing next line of CLI script until the billing invoice section is successfully created.
        text: |-
               az billing invoice section wait --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{invoiceSectionName}" --created
      - name: Pause executing next line of CLI script until the billing invoice section is successfully updated.
        text: |-
               az billing invoice section wait --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{invoiceSectionName}" --updated
"""

helps['billing permission'] = """
    type: group
    short-summary: billing permission
"""

helps['billing permission list'] = """
    type: command
    short-summary: "Lists the billing permissions the caller has on a billing account."
    examples:
      - name: InvoiceSectionPermissionsList
        text: |-
               az billing permission list --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--invoice-section-name "{invoiceSectionName}"
"""

helps['billing subscription'] = """
    type: group
    short-summary: billing subscription
"""

helps['billing subscription list'] = """
    type: command
    short-summary: "Lists the subscriptions for a billing account. The operation is supported for billing accounts \
with agreement type Microsoft Customer Agreement or Microsoft Partner Agreement."
    examples:
      - name: BillingSubscriptionsListByInvoiceSection
        text: |-
               az billing subscription list --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --invoice-section-name "{invoiceSectionName}"
"""

helps['billing subscription show'] = """
    type: command
    short-summary: "Gets a subscription by its ID. The operation is supported for billing accounts with agreement type \
Microsoft Customer Agreement and Microsoft Partner Agreement."
    examples:
      - name: BillingSubscription
        text: |-
               az billing subscription show --account-name "{billingAccountName}"
"""

helps['billing subscription update'] = """
    type: command
    short-summary: "Updates the properties of a billing subscription. Currently, cost center can be updated. The \
operation is supported only for billing accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: UpdateBillingProperty
        text: |-
               az billing subscription update --account-name "{billingAccountName}" --cost-center "ABC1234"
"""

helps['billing subscription move'] = """
    type: command
    short-summary: "Moves a subscription's charges to a new invoice section. The new invoice section must belong to \
the same billing profile as the existing invoice section. This operation is supported for billing accounts with \
agreement type Microsoft Customer Agreement."
    examples:
      - name: MoveBillingSubscription
        text: |-
               az billing subscription move --account-name "{billingAccountName}" --destination-invoice-section-id \
"/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{billingProfileName}/invoiceSections\
/{newInvoiceSectionName}"
"""

helps['billing subscription validate-move'] = """
    type: command
    short-summary: "Validates if a subscription's charges can be moved to a new invoice section. This operation is \
supported for billing accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: SubscriptionMoveValidateFailure
        text: |-
               az billing subscription validate-move --account-name "{billingAccountName}" \
--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{bi\
llingProfileName}/invoiceSections/{newInvoiceSectionName}"
      - name: SubscriptionMoveValidateSuccess
        text: |-
               az billing subscription validate-move --account-name "{billingAccountName}" \
--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{bi\
llingProfileName}/invoiceSections/{newInvoiceSectionName}"
"""

helps['billing subscription wait'] = """
    type: command
    short-summary: Place the CLI in a waiting state until a condition of the billing subscription is met.
    examples:
      - name: Pause executing next line of CLI script until the billing subscription is successfully created.
        text: |-
               az billing subscription wait --account-name "{billingAccountName}" --created
"""

helps['billing product'] = """
    type: group
    short-summary: billing product
"""

helps['billing product list'] = """
    type: command
    short-summary: "Lists the products for a billing account. These don't include products billed based on usage. The \
operation is supported for billing accounts with agreement type Microsoft Customer Agreement or Microsoft Partner \
Agreement."
    examples:
      - name: ProductsListByInvoiceSection
        text: |-
               az billing product list --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--invoice-section-name "{invoiceSectionName}"
"""

helps['billing product show'] = """
    type: command
    short-summary: "Gets a product by ID. The operation is supported only for billing accounts with agreement type \
Microsoft Customer Agreement."
    examples:
      - name: Product
        text: |-
               az billing product show --account-name "{billingAccountName}" --name "{productName}"
"""

helps['billing product update'] = """
    type: command
    short-summary: "Updates the properties of a Product. Currently, auto renew can be updated. The operation is \
supported only for billing accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: UpdateBillingProperty
        text: |-
               az billing product update --account-name "{billingAccountName}" --auto-renew "Off" --name \
"{productName}"
"""

helps['billing product move'] = """
    type: command
    short-summary: "Moves a product's charges to a new invoice section. The new invoice section must belong to the \
same billing profile as the existing invoice section. This operation is supported only for products that are purchased \
with a recurring charge and for billing accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: MoveProduct
        text: |-
               az billing product move --account-name "{billingAccountName}" --destination-invoice-section-id \
"/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{billingProfileName}/invoiceSections\
/{newInvoiceSectionName}" --name "{productName}"
"""

helps['billing product validate-move'] = """
    type: command
    short-summary: "Validates if a product's charges can be moved to a new invoice section. This operation is \
supported only for products that are purchased with a recurring charge and for billing accounts with agreement type \
Microsoft Customer Agreement."
    examples:
      - name: SubscriptionMoveValidateFailure
        text: |-
               az billing product validate-move --account-name "{billingAccountName}" --destination-invoice-section-id \
"/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{billingProfileName}/invoiceSections\
/{newInvoiceSectionName}" --name "{productName}"
      - name: SubscriptionMoveValidateSuccess
        text: |-
               az billing product validate-move --account-name "{billingAccountName}" --destination-invoice-section-id \
"/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{billingProfileName}/invoiceSections\
/{newInvoiceSectionName}" --name "{productName}"
"""

helps['billing invoice'] = """
    type: group
    short-summary: billing invoice
"""

helps['billing invoice list'] = """
    type: command
    short-summary: "Lists the invoices for a subscription."
    examples:
      - name: InvoicesListByBillingProfile
        text: |-
               az billing invoice list --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--period-end-date "2018-06-30" --period-start-date "2018-01-01"
      - name: InvoicesListByBillingProfileWithRebillDetails
        text: |-
               az billing invoice list --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--period-end-date "2018-06-30" --period-start-date "2018-01-01"
"""

helps['billing invoice show'] = """
    type: command
    short-summary: "Gets an invoice by billing account name and ID. The operation is supported for billing accounts \
with agreement type Microsoft Partner Agreement or Microsoft Customer Agreement."
    examples:
      - name: CreditNote
        text: |-
               az billing invoice show --account-name "{billingAccountName}" --name "{invoiceName}"
      - name: Invoice
        text: |-
               az billing invoice show --account-name "{billingAccountName}" --name "{invoiceName}"
      - name: InvoiceWithRebillDetails
        text: |-
               az billing invoice show --account-name "{billingAccountName}" --name "{invoiceName}"
      - name: VoidInvoice
        text: |-
               az billing invoice show --account-name "{billingAccountName}" --name "{invoiceName}"
"""

helps['billing transaction'] = """
    type: group
    short-summary: billing transaction
"""

helps['billing transaction list'] = """
    type: command
    short-summary: "Lists the transactions for an invoice. Transactions include purchases, refunds and Azure usage \
charges."
    examples:
      - name: TransactionsListByInvoice
        text: |-
               az billing transaction list --account-name "{billingAccountName}" --invoice-name "{invoiceName}"
"""

helps['billing policy'] = """
    type: group
    short-summary: billing policy
"""

helps['billing policy update'] = """
    type: command
    short-summary: "Updates the policies for a billing profile. This operation is supported only for billing accounts \
with agreement type Microsoft Customer Agreement."
    examples:
      - name: UpdatePolicy
        text: |-
               az billing policy update --account-name "{billingAccountName}" --profile-name "{billingProfileName}" \
--marketplace-purchases "OnlyFreeAllowed" --reservation-purchases "NotAllowed" --view-charges "Allowed"
"""

helps['billing property'] = """
    type: group
    short-summary: billing property
"""

helps['billing property show'] = """
    type: command
    short-summary: "Get the billing properties for a subscription. This operation is not supported for billing \
accounts with agreement type Enterprise Agreement."
    examples:
      - name: BillingProperty
        text: |-
               az billing property show
"""

helps['billing property update'] = """
    type: command
    short-summary: "Updates the billing property of a subscription. Currently, cost center can be updated. The \
operation is supported only for billing accounts with agreement type Microsoft Customer Agreement."
    examples:
      - name: UpdateBillingProperty
        text: |-
               az billing property update --cost-center "1010"
"""

helps['billing role-definition'] = """
    type: group
    short-summary: billing role-definition
"""

helps['billing role-definition list'] = """
    type: command
    short-summary: "Lists the role definitions for a billing account. The operation is supported for billing accounts \
with agreement type Microsoft Partner Agreement or Microsoft Customer Agreement."
    examples:
      - name: InvoiceSectionRoleDefinitionsList
        text: |-
               az billing role-definition list --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --invoice-section-name "{invoiceSectionName}"
"""

helps['billing role-assignment'] = """
    type: group
    short-summary: billing role-assignment
"""

helps['billing role-assignment list'] = """
    type: command
    short-summary: "Lists the role assignments for the caller on a billing account. The operation is supported for \
billing accounts with agreement type Microsoft Partner Agreement or Microsoft Customer Agreement."
    examples:
      - name: InvoiceSectionRoleAssignmentList
        text: |-
               az billing role-assignment list --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --invoice-section-name "{invoiceSectionName}"
"""

helps['billing role-assignment delete'] = """
    type: command
    short-summary: "Deletes a role assignment for the caller on a billing account. The operation is supported for \
billing accounts with agreement type Microsoft Partner Agreement or Microsoft Customer Agreement."
    examples:
      - name: InvoiceSectionRoleAssignmentDelete
        text: |-
               az billing role-assignment delete --account-name "{billingAccountName}" --profile-name \
"{billingProfileName}" --name "{billingRoleAssignmentName}" --invoice-section-name "{invoiceSectionName}"
"""

helps['billing agreement'] = """
    type: group
    short-summary: billing agreement
"""

helps['billing agreement list'] = """
    type: command
    short-summary: "Lists the agreements for a billing account."
    examples:
      - name: AgreementsListByBillingAccount
        text: |-
               az billing agreement list --account-name "{billingAccountName}"
"""

helps['billing agreement show'] = """
    type: command
    short-summary: "Gets an agreement by ID."
    examples:
      - name: AgreementByName
        text: |-
               az billing agreement show --name "{agreementName}" --account-name "{billingAccountName}"
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import struct
import sys
from knack.log import get_logger
from msrestazure.tools import parse_resource_id
from azure.cli.core.azclierror import (
    AzureConnectionError,
    ValidationError,
    CLIInternalError
)
from azure.cli.core.extension.operations import _install_deps_for_psycopg2, _run_pip
from azure.cli.core.profiles import ResourceType
from azure.cli.core._profile import Profile
from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.util import random_string
from azure.cli.core.commands import LongRunningOperation
from azure.cli.core.commands.arm import ArmTemplateBuilder
from ._utils import run_cli_cmd, generate_random_string, is_packaged_installed
from ._resource_config import (
    RESOURCE,
)
from ._validators import (
    get_source_resource_name,
    get_target_resource_name,
)

logger = get_logger(__name__)


# pylint: disable=line-too-long
# For db(mysqlFlex/psql/psqlFlex/sql) linker with auth type=systemAssignedIdentity, enable AAD auth and create db user on data plane
# For other linker, ignore the steps
def enable_mi_for_db_linker(cmd, source_id, target_id, auth_info, client_type, connection_name):
    # return if connection is not for db mi
    if auth_info['auth_type'] not in {'systemAssignedIdentity'}:
        return

    source_type = get_source_resource_name(cmd)
    target_type = get_target_resource_name(cmd)
    source_handler = getSourceHandler(source_id, source_type)
    if source_handler is None:
        return
    target_handler = getTargetHandler(
        cmd, target_id, target_type, auth_info['auth_type'], client_type, connection_name)
    if target_handler is None:
        return

    user_info = run_cli_cmd(
        'az ad user show --id {}'.format(target_handler.login_username))
    user_object_id = user_info.get('objectId') if user_info.get('objectId') is not None \
        else user_info.get('id')
    if user_object_id is None:
        raise Exception(
            "No object id found for user {}".format(target_handler.login_username))

    # enable source mi
    source_object_id = source_handler.get_identity_pid()

    identity_info = run_cli_cmd(
        'az ad sp show --id {}'.format(source_object_id), 15, 10)
    client_id = identity_info.get('appId')
    identity_name = identity_info.get('displayName')

    # enable target aad authentication and set login user as db aad admin
    target_handler.enable_target_aad_auth()
    target_handler.set_user_admin(
        user_object_id, mysql_identity_id=auth_info.get('mysql-identity-id'))

    # create an aad user in db
    target_handler.create_aad_user(identity_name, client_id)
    return target_handler.get_auth_config()


# pylint: disable=no-self-use, unused-argument, too-many-instance-attributes
def getTargetHandler(cmd, target_id, target_type, auth_type, client_type, connection_name):
    if target_type in {RESOURCE.Sql}:
        return SqlHandler(cmd, target_id, target_type, auth_type, connection_name)
    if target_type in {RESOURCE.Postgres}:
        return PostgresSingleHandler(cmd, target_id, target_type, auth_type, connection_name)
    if target_type in {RESOURCE.PostgresFlexible}:
        return PostgresFlexHandler(cmd, target_id, target_type, auth_type, connection_name)
    if target_type in {RESOURCE.MysqlFlexible}:
        return MysqlFlexibleHandler(cmd, target_id, target_type, auth_type, connection_name)
    return None


class TargetHandler:
    target_id = ""
    target_type = ""
    profile = None
    cmd = None
    tenant_id = ""
    subscription = ""
    resource_group = ""
    login_username = ""
    endpoint = ""
    aad_username = ""

    auth_type = ""

    def __init__(self, cmd, target_id, target_type, auth_type, connection_name):
        self.profile = Profile(cli_ctx=cmd.cli_ctx)
        self.cmd = cmd
        self.target_id = target_id
        self.target_type = target_type
        self.aad_username = "aad_" + connection_name
        self.tenant_id = Profile(
            cli_ctx=cmd.cli_ctx).get_subscription().get("tenantId")
        target_segments = parse_resource_id(target_id)
        self.subscription = target_segments.get('subscription')
        self.resource_group = target_segments.get('resource_group')
        self.auth_type = auth_type
        self.login_username = run_cli_cmd(
            'az account show').get("user").get("name")

    def enable_target_aad_auth(self):
        return

    def set_user_admin(self, user_object_id, **kwargs):
        return

    def set_target_firewall(self, add_new_rule, ip_name):
        return

    def create_aad_user(self, identity_name, client_id):
        return

    def get_auth_config(self):
        return


class MysqlFlexibleHandler(TargetHandler):

    server = ""
    dbname = ""

    def __init__(self, cmd, target_id, target_type, auth_type, connection_name):
        super().__init__(cmd, target_id, target_type, auth_type, connection_name)
        self.endpoint = cmd.cli_ctx.cloud.suffixes.mysql_server_endpoint
        target_segments = parse_resource_id(target_id)
        self.server = target_segments.get('name')
        self.dbname = target_segments.get('child_name_1')

    def set_user_admin(self, user_object_id, **kwargs):
        mysql_identity_id = kwargs['mysql_identity_id']
        admins = run_cli_cmd(
            'az mysql flexible-server ad-admin list -g {} -s {} --subscription {}'.format(
                self.resource_group, self.server, self.subscription)
        )
        is_admin = any(ad.get('sid') == user_object_id for ad in admins)
        if is_admin:
            return

        logger.warning('Set current user as DB Server AAD Administrators.')
        # set user as AAD admin
        if mysql_identity_id is None:
            raise ValidationError(
                "Provide '--system-identity mysql-identity-id=xx' to set {} as AAD administrator.".format(self.user))
        mysql_umi = run_cli_cmd(
            'az mysql flexible-server identity list -g {} -s {} --subscription {}'.format(self.resource_group, self.server, self.subscription))
        if (not mysql_umi) or mysql_identity_id not in mysql_umi.get("userAssignedIdentities"):
            run_cli_cmd('az mysql flexible-server identity assign -g {} -s {} --subscription {} --identity {}'.format(
                self.resource_group, self.server, self.subscription, mysql_identity_id))
        run_cli_cmd('az mysql flexible-server ad-admin create -g {} -s {} --subscription {} -u {} -i {} --identity {}'.format(
            self.resource_group, self.server, self.subscription, self.login_username, user_object_id, mysql_identity_id))

    def create_aad_user(self, identity_name, client_id):
        query_list = self.get_create_query(client_id)
        connection_kwargs = self.get_connection_string()
        ip_name = None
        try:
            logger.warning("Connecting to database...")
            self.create_aad_user_in_mysql(connection_kwargs, query_list)
        except AzureConnectionError:
            # allow public access
            ip_name = generate_random_string(prefix='svc_').lower()
            self.set_target_firewall(True, ip_name)
            # create again
            self.create_aad_user_in_mysql(connection_kwargs, query_list)

        # remove firewall rule
        if ip_name is not None:
            try:
                self.set_target_firewall(False, ip_name)
            # pylint: disable=bare-except
            except:
                pass
                # logger.warning('Please manually delete firewall rule %s to avoid security issue', ipname)

    def set_target_firewall(self, add_new_rule, ip_name):
        if add_new_rule:
            target = run_cli_cmd(
                'az mysql flexible-server show --ids {}'.format(self.target_id))
            # logger.warning("Update database server firewall rule to connect...")
            if target.get('network').get('publicNetworkAccess') == "Disabled":
                return
            run_cli_cmd(
                'az mysql flexible-server firewall-rule create --resource-group {0} --name {1} --rule-name {2} '
                '--subscription {3} --start-ip-address 0.0.0.0 --end-ip-address 255.255.255.255'.format(
                    self.resource_group, self.server, ip_name, self.subscription)
            )
        # logger.warning("Remove database server firewall rules to recover...")
        # run_cli_cmd('az mysql server firewall-rule delete -g {0} -s {1} -n {2} -y'.format(rg, server, ipname))
        # if deny_public_access:
        #     run_cli_cmd('az mysql server update --public Disabled --ids {}'.format(target_id))

    def create_aad_user_in_mysql(self, connection_kwargs, query_list):
        if not is_packaged_installed('pymysql'):
            _run_pip(["install", "pymysql"])
        # pylint: disable=import-error
        try:
            import pymysql
            from pymysql.constants import CLIENT
        except ModuleNotFoundError:
            raise CLIInternalError("Dependency pymysql can't be installed, please install it manually with `" + sys.executable + " -m pip install pymysql`.")

        connection_kwargs['client_flag'] = CLIENT.MULTI_STATEMENTS
        try:
            connection = pymysql.connect(**connection_kwargs)
            cursor = connection.cursor()
            for q in query_list:
                try:
                    logger.debug(q)
                    cursor.execute(q)
                except Exception as e:  # pylint: disable=broad-except
                    logger.warning(
                        "Query %s, error: %s", q, str(e))
        except pymysql.Error as e:
            raise AzureConnectionError("Fail to connect mysql. " + str(e))
        if cursor is not None:
            try:
                cursor.close()
            except Exception as e:  # pylint: disable=broad-except
                raise CLIInternalError(str(e))

    def get_connection_string(self):
        password = run_cli_cmd(
            'az account get-access-token --resource-type oss-rdbms').get('accessToken')

        return {
            'host': self.server + self.endpoint,
            'database': self.dbname,
            'user': self.login_username,
            'password': password,
            'ssl': {"fake_flag_to_enable_tls": True},
            'autocommit': True
        }

    def get_create_query(self, client_id):
        return [
            "SET aad_auth_validate_oids_in_tenant = OFF;",
            "DROP USER IF EXISTS '{}'@'%';".format(self.aad_username),
            "CREATE AADUSER '{}' IDENTIFIED BY '{}';".format(
                self.aad_username, client_id),
            "GRANT ALL PRIVILEGES ON `{}`.* TO '{}'@'%';".format(
                self.dbname, self.aad_username),
            "FLUSH privileges;"
        ]

    def get_auth_config(self):
        if self.auth_type in {'systemAssignedIdentity'}:
            return {
                'auth_type': 'secret',
                'name': self.aad_username,
                'secret_info': {
                    'secret_type': 'rawValue'
                }
            }


class SqlHandler(TargetHandler):

    server = ""
    dbname = ""

    def __init__(self, cmd, target_id, target_type, auth_type, connection_name):
        super().__init__(cmd, target_id, target_type, auth_type, connection_name)
        self.endpoint = cmd.cli_ctx.cloud.suffixes.sql_server_hostname
        target_segments = parse_resource_id(target_id)
        self.server = target_segments.get('name')
        self.dbname = target_segments.get('child_name_1')

    def set_user_admin(self, user_object_id, **kwargs):
        # pylint: disable=not-an-iterable
        admins = run_cli_cmd(
            'az sql server ad-admin list --ids {}'.format(self.target_id))
        is_admin = any(ad.get('sid') == user_object_id for ad in admins)
        if not is_admin:
            logger.warning('Setting current user as database server AAD admin:'
                           ' user=%s object id=%s', self.login_username, user_object_id)
            run_cli_cmd('az sql server ad-admin create -g {} --server-name {} --display-name {} --object-id {} --subscription {}'.format(
                self.resource_group, self.server, self.login_username, user_object_id, self.subscription)).get('objectId')

    def create_aad_user(self, identity_name, client_id):
        self.aad_username = identity_name

        query_list = self.get_create_query(client_id)
        connection_args = self.get_connection_string()
        ip_name = None
        try:
            logger.warning("Connecting to database...")
            self.create_aad_user_in_sql(connection_args, query_list)
        except AzureConnectionError:
            # allow public access
            ip_name = generate_random_string(prefix='svc_').lower()
            self.set_target_firewall(True, ip_name)
            # create again
            self.create_aad_user_in_sql(connection_args, query_list)

        # remove firewall rule
        if ip_name is not None:
            try:
                self.set_target_firewall(False, ip_name)
            # pylint: disable=bare-except
            except:
                pass
                # logger.warning('Please manually delete firewall rule %s to avoid security issue', ipname)

    def set_target_firewall(self, add_new_rule, ip_name):
        if add_new_rule:
            target = run_cli_cmd(
                'az sql server show --ids {}'.format(self.target_id))
            # logger.warning("Update database server firewall rule to connect...")
            if target.get('publicNetworkAccess') == "Disabled":
                run_cli_cmd(
                    'az sql server update -e true --ids {}'.format(self.target_id))
            run_cli_cmd(
                'az sql server firewall-rule create -g {0} -s {1} -n {2} '
                '--subscription {3} --start-ip-address 0.0.0.0 --end-ip-address 255.255.255.255'.format(
                    self.resource_group, self.server, ip_name, self.subscription)
            )
            return False

    def create_aad_user_in_sql(self, connection_args, query_list):

        if not is_packaged_installed('pyodbc'):
            _run_pip(["install", "pyodbc"])

        # pylint: disable=import-error, c-extension-no-member
        try:
            import pyodbc
        except ModuleNotFoundError:
            raise CLIInternalError(
                "Dependency pyodbc can't be installed, please install it manually with `" + sys.executable + " -m pip install pyodbc`.")
        drivers = [x for x in pyodbc.drivers() if x == 'ODBC Driver 18 for SQL Server']
        if not drivers:
            raise CLIInternalError(
                "Please manually install odbc 18 for SQL server, reference: https://docs.microsoft.com/en-us/sql/connect/odbc/download-odbc-driver-for-sql-server?view=sql-server-ver16")
        try:
            with pyodbc.connect(connection_args.get("connection_string"), attrs_before=connection_args.get("attrs_before")) as conn:
                with conn.cursor() as cursor:
                    for execution_query in query_list:
                        try:
                            cursor.execute(execution_query)
                        except pyodbc.ProgrammingError as e:
                            logger.warning(e)
                        conn.commit()
        except pyodbc.Error as e:
            raise AzureConnectionError("Fail to connect sql. " + str(e))

    def get_connection_string(self):
        token_bytes = run_cli_cmd(
            'az account get-access-token --output json --resource https://database.windows.net/').get('accessToken').encode('utf-16-le')

        token_struct = struct.pack(f'<I{len(token_bytes)}s', len(token_bytes), token_bytes)
        # This connection option is defined by microsoft in msodbcsql.h
        SQL_COPT_SS_ACCESS_TOKEN = 1256
        conn_string = 'DRIVER={ODBC Driver 18 for SQL Server};server=' + \
            self.server + self.endpoint + ';database=' + self.dbname + ';'
        return {'connection_string': conn_string, 'attrs_before': {SQL_COPT_SS_ACCESS_TOKEN: token_struct}}

    def get_create_query(self, client_id):
        role_q = "CREATE USER \"{}\" FROM EXTERNAL PROVIDER;".format(
            self.aad_username)
        grant_q = "GRANT CONTROL ON DATABASE::{} TO \"{}\";".format(
            self.dbname, self.aad_username)

        return [role_q, grant_q]


class PostgresFlexHandler(TargetHandler):

    db_server = ""
    host = ""
    dbname = ""
    ip = ""

    def __init__(self, cmd, target_id, target_type, auth_type, connection_name):
        super().__init__(cmd, target_id, target_type, auth_type, connection_name)
        self.endpoint = cmd.cli_ctx.cloud.suffixes.postgresql_server_endpoint
        target_segments = parse_resource_id(target_id)
        self.db_server = target_segments.get('name')
        self.host = self.db_server + self.endpoint
        self.dbname = target_segments.get('child_name_1')

    def enable_pg_extension(self):
        try:
            run_cli_cmd('az postgres flexible-server parameter set -g {} -s {} --subscription {} --name azure.extensions --value uuid-ossp'.format(self.resource_group, self.db_server, self.subscription))
        except Exception as e:  # pylint: disable=broad-except
            logger.warning(e)

    def enable_target_aad_auth(self):
        self.enable_pg_extension()

        rq = 'az rest -u https://management.azure.com/subscriptions/{}/resourceGroups/{}/providers/Microsoft.DBforPostgreSQL/flexibleServers/{}?api-version=2022-03-08-privatepreview'.format(
            self.subscription, self.resource_group, self.db_server)
        server_info = run_cli_cmd(rq)
        if server_info.get("properties").get("authConfig").get("activeDirectoryAuthEnabled"):
            return
        logger.warning('Enabling Postgres flexible server AAD authentication')
        server = self.db_server
        master_template = ArmTemplateBuilder()
        master_template.add_resource({
            'type': "Microsoft.DBforPostgreSQL/flexibleServers",
            'apiVersion': '2022-03-08-privatepreview',
            'name': server,
            'location': "East US",
            'properties': {
                'authConfig': {
                    'activeDirectoryAuthEnabled': True,
                    'tenantId': self.tenant_id
                },
                'createMode': "Update"
            },
        })

        template = master_template.build()
        # parameters = master_template.build_parameters()

        # deploy ARM template
        cmd = self.cmd
        deployment_name = 'pg_deploy_' + random_string(32)
        client = get_mgmt_service_client(
            cmd.cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES).deployments
        DeploymentProperties = cmd.get_models(
            'DeploymentProperties', resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)
        properties = DeploymentProperties(
            template=template, parameters={}, mode='incremental')
        Deployment = cmd.get_models(
            'Deployment', resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)
        deployment = Deployment(properties=properties)

        LongRunningOperation(cmd.cli_ctx)(
            client.begin_create_or_update(self.resource_group, deployment_name, deployment))

    def set_user_admin(self, user_object_id, **kwargs):
        rq = 'az rest -u https://management.azure.com/subscriptions/{}/resourceGroups/{}/providers/Microsoft.DBforPostgreSQL/flexibleServers/{}/administrators?api-version=2022-03-08-privatepreview'.format(
            self.subscription, self.resource_group, self.db_server)
        admins = run_cli_cmd(rq).get("value")
        is_admin = any(user_object_id in u.get(
            "properties").get("objectId") for u in admins)
        if is_admin:
            return
        logger.warning('Set current user as DB Server AAD Administrators.')
        cmd = self.cmd
        master_template = ArmTemplateBuilder()
        master_template.add_resource({
            'type': "Microsoft.DBforPostgreSQL/flexibleServers/administrators",
            'apiVersion': '2022-03-08-privatepreview',
            'name': self.db_server + "/" + user_object_id,
            'location': "East US",
            'properties': {
                'principalName': self.login_username,
                'principalType': 'User',
                'tenantId': self.tenant_id,
                'createMode': "Update"
            },
        })

        template = master_template.build()
        # parameters = master_template.build_parameters()

        # deploy ARM template
        deployment_name = 'pg_addAdmins_' + random_string(32)
        client = get_mgmt_service_client(
            cmd.cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES).deployments
        DeploymentProperties = cmd.get_models(
            'DeploymentProperties', resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)
        properties = DeploymentProperties(
            template=template, parameters={}, mode='incremental')
        Deployment = cmd.get_models(
            'Deployment', resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)
        deployment = Deployment(properties=properties)

        LongRunningOperation(cmd.cli_ctx)(
            client.begin_create_or_update(self.resource_group, deployment_name, deployment))

    def create_aad_user(self, identity_name, client_id):
        # self.aad_user = identity_name or self.aad_user

        query_list = self.get_create_query(client_id)
        connection_string = self.get_connection_string()
        ip_name = None
        try:
            logger.warning("Connecting to database...")
            self.create_aad_user_in_pg(connection_string, query_list)
        except AzureConnectionError:
            # allow public access
            ip_name = generate_random_string(prefix='svc_').lower()
            self.set_target_firewall(True, ip_name)
            # create again
            self.create_aad_user_in_pg(connection_string, query_list)

        # remove firewall rule
        if ip_name is not None:
            try:
                self.set_target_firewall(False, ip_name)
            # pylint: disable=bare-except
            except:
                pass
                # logger.warning('Please manually delete firewall rule %s to avoid security issue', ipname)

    def set_target_firewall(self, add_new_rule, ip_name):
        if add_new_rule:
            target = run_cli_cmd(
                'az postgres flexible-server show --ids {}'.format(self.target_id))
            # logger.warning("Update database server firewall rule to connect...")
            if target.get('network').get('publicNetworkAccess') == "Disabled":
                return True
            start_ip = self.ip or '0.0.0.0'
            end_ip = self.ip or '255.255.255.255'
            run_cli_cmd(
                'az postgres flexible-server firewall-rule create --resource-group {0} --name {1} --rule-name {2} '
                '--subscription {3} --start-ip-address {4} --end-ip-address {5}'.format(
                    self.resource_group, self.db_server, ip_name, self.subscription, start_ip, end_ip)
            )
            return False
        # logger.warning("Remove database server firewall rules to recover...")
        # run_cli_cmd('az postgres server firewall-rule delete -g {0} -s {1} -n {2} -y'.format(rg, server, ipname))
        # if deny_public_access:
        #     run_cli_cmd('az postgres server update --public Disabled --ids {}'.format(target_id))

    def create_aad_user_in_pg(self, conn_string, query_list):
        if not is_packaged_installed('psycopg2'):
            _install_deps_for_psycopg2()
            _run_pip(["install", "psycopg2-binary"])
        # pylint: disable=import-error
        try:
            import psycopg2
        except ModuleNotFoundError:
            raise CLIInternalError(
                "Dependency psycopg2 can't be installed, please install it manually with `" + sys.executable + " -m pip install psycopg2-binary`.")

        # pylint: disable=protected-access
        try:
            conn = psycopg2.connect(conn_string)
        except (psycopg2.Error, psycopg2.OperationalError) as e:
            import re
            logger.warning(e)
            search_ip = re.search(
                'no pg_hba.conf entry for host "(.*)", user ', str(e))
            if search_ip is not None:
                self.ip = search_ip.group(1)
            raise AzureConnectionError(
                "Fail to connect to postgresql. " + str(e))

        conn.autocommit = True
        cursor = conn.cursor()
        logger.warning("Adding new AAD user %s to database...",
                       self.aad_username)
        for execution_query in query_list:
            try:
                logger.debug(execution_query)
                cursor.execute(execution_query)
            except psycopg2.Error as e:  # role "aad_user" already exists
                logger.warning(e)

        # Clean up
        conn.commit()
        cursor.close()
        conn.close()

    def get_connection_string(self):
        password = run_cli_cmd(
            'az account get-access-token --resource-type oss-rdbms').get('accessToken')

        # extension functions require the extension to be available, which is the case for postgres (default) database.
        conn_string = "host={} user={} dbname=postgres password={} sslmode=require".format(
            self.host, self.login_username, password)
        return conn_string

    def get_create_query(self, client_id):
        return [
            'drop role IF EXISTS "{0}";'.format(self.aad_username),
            "select * from pgaadauth_create_principal_with_oid('{0}', '{1}', 'ServicePrincipal', false, false);".format(
                self.aad_username, client_id),
            'GRANT ALL PRIVILEGES ON DATABASE "{0}" TO "{1}";'.format(
                self.dbname, self.aad_username),
            'GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO "{}";'.format(
                self.aad_username),
            'GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO "{}";'.format(
                self.aad_username)]

    def get_auth_config(self):
        if self.auth_type in {'systemAssignedIdentity'}:
            return {
                'auth_type': 'secret',
                'name': self.aad_username,
                'secret_info': {
                    'secret_type': 'rawValue'
                }
            }


class PostgresSingleHandler(PostgresFlexHandler):

    def enable_target_aad_auth(self):
        return

    def set_user_admin(self, user_object_id, **kwargs):
        sub = self.subscription
        rg = self.resource_group
        server = self.db_server
        is_admin = True

        # pylint: disable=not-an-iterable
        admins = run_cli_cmd(
            'az postgres server ad-admin list --ids {}'.format(self.target_id))
        is_admin = any(ad.get('sid') == user_object_id for ad in admins)
        if not is_admin:
            logger.warning('Setting current user as database server AAD admin:'
                           ' user=%s object id=%s', self.login_username, user_object_id)
            run_cli_cmd('az postgres server ad-admin create -g {} --server-name {} --display-name {} --object-id {}'
                        ' --subscription {}'.format(rg, server, self.login_username, user_object_id, sub)).get('objectId')

    def set_target_firewall(self, add_new_rule, ip_name):
        sub = self.subscription
        rg = self.resource_group
        server = self.db_server
        target_id = self.target_id
        if add_new_rule:
            target = run_cli_cmd(
                'az postgres server show --ids {}'.format(target_id))
            # logger.warning("Update database server firewall rule to connect...")
            if target.get('publicNetworkAccess') == "Disabled":
                run_cli_cmd(
                    'az postgres server update --public Enabled --ids {}'.format(target_id))
            start_ip = self.ip or '0.0.0.0'
            end_ip = self.ip or '255.255.255.255'
            run_cli_cmd(
                'az postgres server firewall-rule create -g {0} -s {1} -n {2} --subscription {3}'
                ' --start-ip-address {4} --end-ip-address {5}'.format(
                    rg, server, ip_name, sub, start_ip, end_ip)
            )
            return target.get('publicNetworkAccess') == "Disabled"
        # logger.warning("Remove database server firewall rules to recover...")
        # run_cli_cmd('az postgres server firewall-rule delete -g {0} -s {1} -n {2} -y'.format(rg, server, ipname))
        # if deny_public_access:
        #     run_cli_cmd('az postgres server update --public Disabled --ids {}'.format(target_id))

    def get_connection_string(self):
        password = run_cli_cmd('az account get-access-token --resource-type oss-rdbms').get('accessToken')

        # extension functions require the extension to be available, which is the case for postgres (default) database.
        conn_string = "host={} user={} dbname=postgres password={} sslmode=require".format(
            self.host, self.login_username + '@' + self.db_server, password)
        return conn_string

    def get_create_query(self, client_id):

        return [
            'SET aad_validate_oids_in_tenant = off;',
            'drop role IF EXISTS "{0}";'.format(self.aad_username),
            "CREATE ROLE {0} WITH LOGIN PASSWORD '{1}' IN ROLE azure_ad_user;".format(
                self.aad_username, client_id),
            'GRANT ALL PRIVILEGES ON DATABASE "{0}" TO "{1}";'.format(
                self.dbname, self.aad_username),
            'GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO "{}";'.format(
                self.aad_username),
            'GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO "{}";'.format(
                self.aad_username)
        ]


def getSourceHandler(source_id, source_type):
    if source_type in {RESOURCE.WebApp}:
        return WebappHandler(source_id, source_type)
    if source_type in {RESOURCE.ContainerApp}:
        return ContainerappHandler(source_id, source_type)
    if source_type in {RESOURCE.SpringCloud, RESOURCE.SpringCloudDeprecated}:
        return SpringHandler(source_id, source_type)


# pylint: disable=too-few-public-methods
class SourceHandler:
    source_id = ""
    source_type = ""

    def __init__(self, source_id, source_type: RESOURCE):
        self.source_id = source_id
        self.source_type = source_type.value

    def get_identity_pid(self):
        return


def output_is_none(output):
    return not output.stdout


class SpringHandler(SourceHandler):
    def get_identity_pid(self):
        segments = parse_resource_id(self.source_id)
        sub = segments.get('subscription')
        spring = segments.get('name')
        app = segments.get('child_name_1')
        rg = segments.get('resource_group')
        logger.warning(
            'Checking if Spring app enables System Identity...')
        identity = run_cli_cmd('az {} app identity show -g {} -s {} -n {} --subscription {}'.format(
            self.source_type, rg, spring, app, sub))
        if (identity is None or identity.get('type') != "SystemAssigned"):
            # assign system identity for spring-cloud
            logger.warning('Enabling Spring app System Identity...')
            run_cli_cmd(
                'az {} app identity assign -g {} -s {} -n {} --subscription {}'.format(
                    self.source_type, rg, spring, app, sub))

            identity = run_cli_cmd('az {} app identity show -g {} -s {} -n {} --subscription {}'.format(
                self.source_type, rg, spring, app, sub), 15, 5, output_is_none)

        if identity is None:
            raise CLIInternalError(
                "Unable to get system identity of Spring. Please try it later.")
        return identity.get('principalId')


class WebappHandler(SourceHandler):
    def get_identity_pid(self):
        logger.warning('Checking if WebApp enables System Identity...')
        identity = run_cli_cmd(
            'az webapp identity show --ids {}'.format(self.source_id))
        if (identity is None or "SystemAssigned" not in identity.get('type')):
            # assign system identity for spring-cloud
            logger.warning('Enabling WebApp System Identity...')
            run_cli_cmd(
                'az webapp identity assign --ids {}'.format(self.source_id))

            identity = run_cli_cmd(
                'az webapp identity show --ids {}'.format(self.source_id), 15, 5, output_is_none)

        if identity is None:
            raise CLIInternalError(
                "Unable to get system identity of Web App. Please try it later.")
        return identity.get('principalId')


class ContainerappHandler(SourceHandler):
    def get_identity_pid(self):
        logger.warning('Checking if Container App enables System Identity...')
        identity = run_cli_cmd(
            'az containerapp identity show --ids {}'.format(self.source_id))
        if (identity is None or "SystemAssigned" not in identity.get('type')):
            # assign system identity for spring-cloud
            logger.warning('Enabling Container App System Identity...')
            run_cli_cmd(
                'az containerapp identity assign --ids {} --system-assigned'.format(self.source_id))
            identity = run_cli_cmd(
                'az containerapp identity show --ids {}'.format(self.source_id), 15, 5, output_is_none)

        if identity is None:
            raise CLIInternalError(
                "Unable to get system identity of Container App. Please try it later.")
        return identity.get('principalId')
